import arg_parser
import command_loader
import help

import importlib
from types import ModuleType
from typing import List

def run(args: List[str]) -> None:

    command_paths: list = command_loader.commands_lookup()
    parsed_args: list = arg_parser.parse(args[1:], command_paths)
    modules: List[ModuleType] = []

    # load all available command modules
    for path in command_paths:
        module_string = path.split('./')[1].replace('/', '.').replace('.py', '')
        modules.append(importlib.import_module(module_string))

    if 'py' in parsed_args[0]:
        # convert directory string into the module name
        cmd = parsed_args[0].split('./')[1].replace('/', '.').split('.py')[0]
        for m in modules:
            if m.__name__ == cmd:
                m.run(parsed_args[1:])

    else:
        help.commands(args[0], modules, parsed_args[0])


from types import ModuleType
from typing import List

def commands(app: str, modules: List[ModuleType], path: str) -> None:
    # when no topic is found
    if path == '':
        topic = 'commands'
        print(f'usage: {app} [-h] command',end="\n\n")
    else:
        # produce format for 'topic': topic1.topic2.topic3
        topic = path.split('./commands/')[1].replace('/', '.')
        print(f'usage: {app} {topic.replace(".", " ")} [-h] command',end="\n\n")
    
    print("commands:")

    cmd_pointer = "" # this helps print the help for the first command in a sub topic
    for m in modules:
        if topic in m.__name__:
            module_elements = m.__name__.split('.')
            
            # deal only with the commands inside the target topic
            i = module_elements.index(topic.split('.')[-1]) # the topic is the deepest folder
            cmd_name = module_elements[i + 1]

            if not cmd_pointer == cmd_name:
                print('  {0:20}  {1}'.format(cmd_name, m.description))

            cmd_pointer = cmd_name


import os

from typing import List

def parse(args: List[str], command_files: List[str]) -> List[str]:
    
    command_path_arg = ""
    args_start = 0

    for arg_position in range(len(args)):

        path = "./commands/" + '/'.join(args[:arg_position + 1])

        if path + ".py" in command_files:
            command_path_arg = path + '.py'
            args_start = arg_position + 1
            break

        elif any(path in substring for substring in command_files):
            if os.path.exists(path):
                command_path_arg = path
                args_start = arg_position + 1

        else:
            break

    return [command_path_arg] + args[args_start:]


import pytest
from pathlib import Path

import clidir


def test_no_args_shows_help(capsys):
    clidir.run(['cli'])
    
    captured = capsys.readouterr()
    assert "cli [-h] command" in captured.out


def test_invalid_command_shows_help(capsys):
    clidir.run(['cli', 'invalid', 'command'])
    
    captured = capsys.readouterr()
    assert "cli [-h] command" in captured.out


@pytest.fixture
def base_path() -> Path:
    """Get the current folder of the test"""
    return Path(__file__).parent


def test_subcommand_no_args_shows_help(base_path: Path, monkeypatch: pytest.MonkeyPatch, capsys):
    monkeypatch.chdir(base_path)
    
    clidir.run(['cli', 'hello'])
    captured = capsys.readouterr()
    assert "cli hello [-h] command" in captured.out

    clidir.run(['cli', 'hello', 'abcd'])
    captured = capsys.readouterr()
    assert "cli hello [-h] command" in captured.out

    clidir.run(['cli', 'hello', 'perso'])
    captured = capsys.readouterr()
    assert "cli hello [-h] command" in captured.out

    with pytest.raises(SystemExit) as e:
        clidir.run(['cli', 'hello', 'person'])

    assert e.type == SystemExit
    assert e.value.code == 2

    captured = capsys.readouterr()
    assert "cli hello person [-h] name" in captured.err


def test_subcommand_valid_args(base_path: Path, monkeypatch: pytest.MonkeyPatch, capsys):
    monkeypatch.chdir(base_path)
    
    clidir.run(['cli', 'hello', 'person', 'John'])
    captured = capsys.readouterr()
    assert "Hello John" in captured.out


import os

from typing import List

def commands_lookup() -> List[str]:
    
    commands_dir = "./commands/"
    command_files = []

    for (file_path, dir_names, file_names) in os.walk(commands_dir):
        if "__pycache__" in dir_names:
            dir_names.remove("__pycache__")
        
        if ".DS_Store" in file_names:
            file_names.remove(".DS_Store")

        if len(file_names) > 0:
            for f in file_names:
                command_files.append(os.path.join(file_path, f))

    return command_files